@@ -30,6 +30,7 @@
 #include "libwacomint.h"
 #include "util-strings.h"
 #include <linux/input-event-codes.h>
+#include <libevdev/libevdev.h>
 
 #include <assert.h>
 #include <glib.h>
@@ -44,6 +45,7 @@
 #define FEATURES_GROUP "Features"
 #define DEVICE_GROUP "Device"
 #define BUTTONS_GROUP "Buttons"
+#define KEYS_GROUP "Keys"
 
 static WacomClass
 libwacom_class_string_to_enum(const char *class)
@@ -455,8 +457,9 @@ set_button_codes_from_string(WacomDevice *device, char **strvals)
 
 	for (guint i = 0; i < g_hash_table_size(device->buttons); i++) {
 		char key = 'A' + i;
-		int code;
+		int code = -1;
 		WacomButton *button = g_hash_table_lookup(device->buttons, GINT_TO_POINTER(key));
+		const char *str = strvals[i];
 
 		if (!button) {
 			g_error("%s: Button %c is not defined, ignoring all codes\n",
@@ -464,17 +467,22 @@ set_button_codes_from_string(WacomDevice *device, char **strvals)
 			goto out;
 		}
 
-		if (!strvals[i]) {
+		if (!str) {
 			g_error ("%s: Missing EvdevCode for button %d, ignoring all codes\n",
 				 device->name, i);
 			goto out;
+		} else if (g_str_has_prefix(str, "BTN")) {
+			code = libevdev_event_code_from_code_name(str);
+		} else if (!safe_atoi_base(str, &code, 16)) {
+			code = -1;
 		}
 
-		if (!safe_atoi_base (strvals[i], &code, 16) || code < BTN_MISC || code >= BTN_DIGI) {
+		if (code < BTN_MISC || code >= BTN_DIGI) {
 			g_warning ("%s: Invalid EvdevCode %s for button %c, ignoring all codes\n",
-				   device->name, strvals[i], key);
+				   device->name, str, key);
 			goto out;
 		}
+
 		button->code = code;
 	}
 	success = true;
@@ -486,6 +494,50 @@ out:
 	return success;
 }
 
+static inline bool
+set_key_codes_from_string(WacomDevice *device, char **strvals)
+{
+	bool success = false;
+	assert(strvals);
+
+	for (unsigned int idx = 0; strvals[idx]; idx++) {
+		const char *str = strvals[idx];
+		int code = -1;
+		int type = -1;
+
+		if (!str) {
+			g_error("%s: Missing KeyCode for key %d, ignoring all codes\n",
+				device->name, idx);
+			goto out;
+		} else if (g_str_has_prefix(str, "KEY")) {
+			type = EV_KEY;
+			code = libevdev_event_code_from_code_name(str);
+		} else if (g_str_has_prefix(str, "SW")) {
+			type = EV_SW;
+			code = libevdev_event_code_from_code_name(str);
+		} else {
+			if (safe_atoi_base (strvals[idx], &code, 16))
+				type = EV_KEY;
+		}
+
+		if (code == -1 || type == -1) {
+			g_warning ("%s: Invalid KeyCode %s, ignoring all codes\n", device->name, str);
+			goto out;
+		}
+
+		device->keycodes[idx].type = type;
+		device->keycodes[idx].code = code;
+		device->num_keycodes = idx + 1;
+	}
+
+	success = true;
+out:
+	if (!success) {
+		memset(device->keycodes, 0, sizeof(device->keycodes));
+	}
+	return success;
+}
+
 static inline void
 set_button_codes_from_heuristics(WacomDevice *device)
 {
@@ -584,6 +636,9 @@ libwacom_parse_buttons(WacomDevice *device,
 {
 	guint i;
 
+	if (!g_key_file_has_group(keyfile, BUTTONS_GROUP))
+		return;
+
 	for (i = 0; i < G_N_ELEMENTS (options); i++)
 		libwacom_parse_buttons_key(device, keyfile, options[i].key, options[i].flag);
 
@@ -594,6 +649,30 @@ libwacom_parse_buttons(WacomDevice *device,
 	device->strips_num_modes = libwacom_parse_num_modes(device, keyfile, "StripsNumModes", WACOM_BUTTON_TOUCHSTRIP_MODESWITCH);
 }
 
+static void
+libwacom_parse_key_codes(WacomDevice *device,
+			 GKeyFile    *keyfile)
+{
+	char **vals;
+
+	vals = g_key_file_get_string_list(keyfile, KEYS_GROUP, "KeyCodes", NULL, NULL);
+	if (vals)
+		set_key_codes_from_string(device, vals);
+
+	g_strfreev (vals);
+}
+
+static void
+libwacom_parse_keys(WacomDevice *device,
+		    GKeyFile    *keyfile)
+{
+	if (!g_key_file_has_group(keyfile, KEYS_GROUP))
+		return;
+
+	libwacom_parse_key_codes(device, keyfile);
+}
+
+
 static int
 styli_id_sort(gconstpointer pa, gconstpointer pb)
 {
@@ -639,6 +718,61 @@ libwacom_parse_styli_list(WacomDeviceDatabase *db, WacomDevice *device,
 	device->styli = array;
 }
 
+static void
+libwacom_parse_features(WacomDevice *device, GKeyFile *keyfile)
+{
+	char **string_list;
+
+	/* Features */
+	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "Stylus", NULL))
+		device->features |= FEATURE_STYLUS;
+
+	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "Touch", NULL))
+		device->features |= FEATURE_TOUCH;
+
+	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "Ring", NULL))
+		device->features |= FEATURE_RING;
+
+	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "Ring2", NULL))
+		device->features |= FEATURE_RING2;
+
+	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "Reversible", NULL))
+		device->features |= FEATURE_REVERSIBLE;
+
+	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "TouchSwitch", NULL))
+		device->features |= FEATURE_TOUCHSWITCH;
+
+	if (device->integration_flags != WACOM_DEVICE_INTEGRATED_UNSET &&
+	    device->integration_flags & WACOM_DEVICE_INTEGRATED_DISPLAY &&
+	    device->features & FEATURE_REVERSIBLE)
+		g_warning ("Tablet '%s' is both reversible and integrated in screen. This is impossible", libwacom_get_match(device));
+
+	if (!(device->features & FEATURE_RING) &&
+	    (device->features & FEATURE_RING2))
+		g_warning ("Tablet '%s' has Ring2 but no Ring. This is impossible", libwacom_get_match(device));
+
+	if (!(device->features & FEATURE_TOUCH) &&
+	    (device->features & FEATURE_TOUCHSWITCH))
+		g_warning ("Tablet '%s' has touch switch but no touch tool. This is impossible", libwacom_get_match(device));
+
+	device->num_strips = g_key_file_get_integer(keyfile, FEATURES_GROUP, "NumStrips", NULL);
+
+	string_list = g_key_file_get_string_list(keyfile, FEATURES_GROUP, "StatusLEDs", NULL, NULL);
+	if (string_list) {
+		guint i, n;
+
+		for (i = 0; string_list[i]; i++) {
+			for (n = 0; n < G_N_ELEMENTS (supported_leds); n++) {
+				if (g_str_equal(string_list[i], supported_leds[n].key)) {
+					g_array_append_val (device->status_leds, supported_leds[n].value);
+					break;
+				}
+			}
+		}
+		g_strfreev (string_list);
+	}
+}
+
 static WacomDevice*
 libwacom_parse_tablet_keyfile(WacomDeviceDatabase *db,
 			      const char *datadir,
@@ -654,7 +788,6 @@ libwacom_parse_tablet_keyfile(WacomDeviceDatabase *db,
 	char *paired;
 	char **string_list;
 	bool success = FALSE;
-	int num_buttons;
 
 	keyfile = g_key_file_new();
 
@@ -758,67 +891,14 @@ libwacom_parse_tablet_keyfile(WacomDeviceDatabase *db,
 		g_array_append_val(device->styli, fallback_stylus);
 	}
 
-	/* Features */
-	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "Stylus", NULL))
-		device->features |= FEATURE_STYLUS;
-
-	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "Touch", NULL))
-		device->features |= FEATURE_TOUCH;
-
-	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "Ring", NULL))
-		device->features |= FEATURE_RING;
-
-	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "Ring2", NULL))
-		device->features |= FEATURE_RING2;
-
-	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "Reversible", NULL))
-		device->features |= FEATURE_REVERSIBLE;
-
-	if (g_key_file_get_boolean(keyfile, FEATURES_GROUP, "TouchSwitch", NULL))
-		device->features |= FEATURE_TOUCHSWITCH;
-
-	if (device->integration_flags != WACOM_DEVICE_INTEGRATED_UNSET &&
-	    device->integration_flags & WACOM_DEVICE_INTEGRATED_DISPLAY &&
-	    device->features & FEATURE_REVERSIBLE)
-		g_warning ("Tablet '%s' is both reversible and integrated in screen. This is impossible", libwacom_get_match(device));
-
-	if (!(device->features & FEATURE_RING) &&
-	    (device->features & FEATURE_RING2))
-		g_warning ("Tablet '%s' has Ring2 but no Ring. This is impossible", libwacom_get_match(device));
-
-	if (!(device->features & FEATURE_TOUCH) &&
-	    (device->features & FEATURE_TOUCHSWITCH))
-		g_warning ("Tablet '%s' has touch switch but no touch tool. This is impossible", libwacom_get_match(device));
-
 	device->num_strips = g_key_file_get_integer(keyfile, FEATURES_GROUP, "NumStrips", NULL);
-
-	num_buttons = g_key_file_get_integer(keyfile, FEATURES_GROUP, "Buttons", &error);
-	if (num_buttons == 0 &&
-	    g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-		g_warning ("Tablet '%s' has no buttons defined, do something!", libwacom_get_match(device));
-		g_clear_error (&error);
-	}
-
 	device->buttons = g_hash_table_new_full(g_direct_hash, g_direct_equal,
 						NULL, g_free);
-	if (num_buttons > 0)
-		libwacom_parse_buttons(device, keyfile);
-
 	device->status_leds = g_array_new (FALSE, FALSE, sizeof(WacomStatusLEDs));
-	string_list = g_key_file_get_string_list(keyfile, FEATURES_GROUP, "StatusLEDs", NULL, NULL);
-	if (string_list) {
-		guint i, n;
 
-		for (i = 0; string_list[i]; i++) {
-			for (n = 0; n < G_N_ELEMENTS (supported_leds); n++) {
-				if (g_str_equal(string_list[i], supported_leds[n].key)) {
-					g_array_append_val (device->status_leds, supported_leds[n].value);
-					break;
-				}
-			}
-		}
-		g_strfreev (string_list);
-	}
+	libwacom_parse_features(device, keyfile);
+	libwacom_parse_buttons(device, keyfile);
+	libwacom_parse_keys(device, keyfile);
 
 	success = TRUE;
 
