@@ -327,7 +327,7 @@ static void setup_tests(WacomDevice *device)
 	WacomClass cls;
 
 	name = libwacom_get_name(device);
-	if (strcmp(name, "Generic") == 0)
+	if (g_str_equal(name, "Generic"))
 		return;
 
 	add_test(device, test_class);
